"""Visuals
  visuals.py 
  Handle all screen visuals and effects.
  Simon May aka may88
  Verison 0.1.0
  """
# Verison History
# 0.1.0  27/06/10  initial version.

import data, jukebox, player, defs
import pygame

screenSize=screenX,screenY=defs.SCREENSIZE
buttons={}
sliders={}
screen=background=volume=repeat=None
paused=False


#
# arg1 = stem of slider image names.
# arg2 = slider name.
class Slider():
  def __init__(self,name,sname):
    self.slider=sname
    self.button=Button(name)
    self.image=data.loadimg(name+"Normal.png")
    self.selected=data.loadimg(name+"Selected.png")
    self.rect=self.image.get_rect()
    self.range=self.rect.height-self.button.rect.height
    (self.rect.left,self.rect.top)=(defs.getLoc(self.slider))
    self.button.rect.centery=self.getValue()
    self.button.rect.left=self.rect.left
    self.state=False      # True when dragging
    self.lasty=0          # last centery location of the button.
 
  def update(self):
    (mLeft,mMid,mRight)=pygame.mouse.get_pressed()
    if mLeft:
      (x,y)=pygame.mouse.get_pos()
      if self.rect.collidepoint((x,y)):
        self.state=True
        self.button.rect.centery=y
        if self.button.rect.top < self.rect.top: self.button.rect.top=self.rect.top
        if self.button.rect.bottom > self.rect.bottom: self.button.rect.bottom=self.rect.bottom
      else:                         # mouse dragged off the slider
        if self.state: self.setValue()
    else:                           # dragging ended
      if self.state: self.setValue()
    if not self.lasty == self.button.rect.centery:
      self.lasty=self.button.rect.centery
      screen.blit(self.selected,self.rect)
      screen.blit(self.image,self.rect,(0,0,self.rect.width,self.button.rect.centery-self.rect.top))
#
# returns co-ordinate based on value.
  def getValue(self):
    if self.slider == defs.VOL:
      coord=self.rect.bottom - ( player.volume * self.range )
      #print coord ###
      return coord
#
# sets the value based on the new slider pos.
  def setValue(self):
    self.state=False
    if self.slider == defs.VOL:
      player.volume=(self.rect.bottom - self.button.rect.bottom)/1.0/self.range
      #print player.volume, self.rect.bottom, self.button.rect.bottom, self.range
      player.setVol()
#
# Toggle is a pair of Buttons: name+"On" and name+"OFF"
# inital state defaults to True(On).
class Toggle:
  def __init__(self,name,state=True):
    self.name=name
    self.state=state 
    self.button=Button(name+"On")
    self.imageOnNormal=self.button.image
    self.imageOnClick=self.button.imageClick
    self.imageOnHover=self.button.imageHover
    self.imageOffNormal=data.loadimg(name+"Off.png")
    self.imageOffClick=data.loadimg(name+"OffClick.png")
    self.imageOffHover=data.loadimg(name+"OffHover.png")
    (self.button.rect.left,self.button.rect.top)=defs.getLoc(self.name)
    self.update()
# 
  def toggle(self):
    if self.state: self.state=False
    else: self.state=True
    self.update()
#        
  def update(self):
    if self.state: 
      self.button.imageNormal=self.imageOnNormal
      self.button.imageClick=self.imageOnClick
      self.button.imageHover=self.imageOnHover
    else:  
      self.button.imageNormal=self.imageOffNormal
      self.button.imageClick=self.imageOffClick
      self.button.imageHover=self.imageOffHover
##      
class Button(pygame.sprite.Sprite):
  def __init__(self,name):
    pygame.sprite.Sprite.__init__(self)
    self.image=data.loadimg(name+".png")
    self.rect=self.image.get_rect()
    self.imageNormal=self.image
    self.imageClick=data.loadimg(name+"Click.png")
    self.imageHover=data.loadimg(name+"Hover.png")
  def update(self):
    if self.rect.collidepoint(pygame.mouse.get_pos()):
      (mLeft,mMid,mRight)=pygame.mouse.get_pressed()
      if mLeft:
        self.image=self.imageClick
      else:
        self.image=self.imageHover
    else:
      self.image=self.imageNormal

def init():
  global screen,background,volume,repeat
  screen=pygame.display.set_mode(screenSize) #,pygame.FULLSCREEN)
  background=pygame.Surface(screenSize)
  background.fill((0,0,0))
  screen.blit(background,(0,0))
  init_play()
  
def init_play():
  global screen,background,volume,repeat
  for name in defs.PREV, defs.PLAY, defs.NEXT, defs.EJECT: 
    button=Button(name)
    (button.rect.left,button.rect.top)=defs.getLoc(name)
    buttons[name]=button
  volume=Slider("slider",defs.VOL)
  repeat=Toggle(defs.REPEAT,False)
  buttons[defs.REPEAT]=repeat.button
#-----------------------------
def display(view):
  if view=="main": view=display_main()
  elif view=="play": view=display_play()
  else: view="quit"
  return view
#-----------------------------
def display_main():
  global screen,background
  allSprites=pygame.sprite.Group(buttons.values(),volume.button)
  action="quit"
  alive=True
  clock=pygame.time.Clock()
  while alive:  
    clock.tick(30)
    for event in pygame.event.get():
      if event.type == pygame.QUIT:
        alive=False
      elif event.type == pygame.KEYDOWN:
        if event.key == pygame.K_ESCAPE:
          alive=False
      elif event.type == pygame.MOUSEBUTTONDOWN:
        action=checkClick(pygame.mouse.get_pos())
        print action  ###
        if action == defs.PLAY: player.pause()
        elif action == defs.PREV: player.prev()
        elif action == defs.NEXT: player.next()
        elif action == defs.REPEAT: repeat.toggle()
        elif action == defs.EJECT: 
          player.stop()
          alive=False
      elif event.type == player.ENDMUSIC:
        if repeat.state: player.autostart()
        else: player.next()
    allSprites.clear(screen, background)
    volume.update()
    allSprites.update()  
    allSprites.draw(screen)   
    pygame.display.flip()
  return action
#-----------------------------
def display_play():
  global screen,background,volume,repeat
  allSprites=pygame.sprite.Group(buttons.values(),volume.button)
  action="quit"
  alive=True
  clock=pygame.time.Clock()
  while alive:  
    clock.tick(30)
    for event in pygame.event.get():
      if event.type == pygame.QUIT:
        alive=False
      elif event.type == pygame.KEYDOWN:
        if event.key == pygame.K_ESCAPE:
          alive=False
      elif event.type == pygame.MOUSEBUTTONDOWN:
        action=checkClick(pygame.mouse.get_pos())
        print action  ###
        if action == defs.PLAY: player.pause()
        elif action == defs.PREV: player.prev()
        elif action == defs.NEXT: player.next()
        elif action == defs.REPEAT: repeat.toggle()
        elif action == defs.EJECT: 
          player.stop()
          alive=False
      elif event.type == player.ENDMUSIC:
        if repeat.state: player.autostart()
        else: player.next()
    allSprites.clear(screen, background)
    volume.update()
    allSprites.update()  
    allSprites.draw(screen)   
    pygame.display.flip()
  return action

# check if the click in on a button.
def checkClick((x,y)):
  action=None
  for name,button in buttons.items():
    if button.rect.collidepoint(x,y):
      clicked=True
      action=name
  return action

def exit():
  pass
###fin

